// Rally.cpp: implementation of the Rally class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Rally.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//-------------------------------------------------------------------------------------------------

Rally::Rally (const char *rallyName)
{
  name=rallyName;
  time=0.;
  invalidStage=false;
}

//-------------------------------------------------------------------------------------------------

void Rally::Reset ()
{
  time=0.;
  invalidStage=false;
  for (int i=0;i<stages.size();i++) stages[i].Reset();
}

//-------------------------------------------------------------------------------------------------

bool Rally::Finished () const
{
  bool res = true;
  for (int i=0;i<stages.size();i++) res &= stages[i].driven;
  return res;
}

//-------------------------------------------------------------------------------------------------

bool Rally::NotStarted () const
{
  for (int i=0;i<stages.size();i++) if (stages[i].driven) return false;
  return true;
}

//-------------------------------------------------------------------------------------------------

bool Rally::Dnf () const
{
  for (int i=0;i<stages.size();i++) if (stages[i].Dnf()) return true;
  return false;
}

//-------------------------------------------------------------------------------------------------

bool Rally::LoadConfig (const char *fileName)
{
  CString msg;

  CStdioFile file;
  if (!file.Open(fileName,CFile::modeRead|CFile::typeText))
  {
    msg.Format("Could not open %s (#%i)",fileName,GetLastError());
    OutputDebugString(msg);
    return false;
  }

  CString line;
  int comment;
  int weather=0;

  while (file.ReadString(line))
  {
    if ((comment=line.Find('#'))>=0) line.Delete(comment,line.GetLength()-comment);
    line.TrimLeft(),line.TrimRight();
    if (line.IsEmpty()||line.GetLength()>255) continue;

    char label[256];
    int value;
    if (sscanf(line,"%s %i",label,&value)!=2) continue;

    if (CString(label).CompareNoCase("weather")==0) weather=value;
    else if (CString(label).CompareNoCase("stage")==0) stages.push_back(Stage(value,weather));
  }
  return true;
}

//-------------------------------------------------------------------------------------------------

float Rally::BestTime () const
{
  float sum=0;
  for (int i=0;i<stages.size();i++)
  {
    if (!stages[i].bestTime) return 0;
    sum+=stages[i].best.finish;
  }
  return sum;
}

float Rally::CurrentTime () const
{
  float sum=0;
  for (int i=0;i<stages.size();i++)
  {
    if (!stages[i].Valid()) return 0;
    sum+=stages[i].finish;
  }
  return sum;
}

//-------------------------------------------------------------------------------------------------

void Rally::ClearBestTime ()
{
  for (int i=0;i<stages.size();i++) stages[i].bestTime = false;
}


CString Rally::BestTimePath () const
{
  CString path;
  path.Format("Plugins\\RBRSingleRally\\%s.tim",(LPCTSTR)name);
  return path;
}

bool Rally::SaveBestTime () const
{
  float best = BestTime();
  float current = CurrentTime();

  if (current==0) return false; // rally not finished
  if (best!=0 && current>=best) return false; // no improvement

  CFile file;
  if (!file.Open(BestTimePath(),CFile::modeCreate|CFile::modeWrite)) return false;

  for (int i=0;i<stages.size();i++)
  {
    const StageTime *pst = &stages[i];
    file.Write(pst,sizeof StageTime);
  }
  
  return true;
}


bool Rally::LoadBestTime ()
{
  CFile file;
  if (!file.Open(BestTimePath(),CFile::modeRead,0)) return false;

  for (int i=0;i<stages.size();i++)
  {
    StageTime st;
    if (!file.Read(&st,sizeof StageTime)) return ClearBestTime(),false;

    if (st.index!=stages[i].index) return ClearBestTime(),false;
    if (st.weather!=stages[i].weather) return ClearBestTime(),false;

    stages[i].best = st;
    stages[i].bestTime = true;
  }

  return true;
}

//-------------------------------------------------------------------------------------------------


