

#include "stdafx.h"
#include "SingleRallyPlugin.h"

//-----------------------------------------------------------------------------

SingleRallyPlugin::SingleRallyPlugin (IRBRGame* pGame)	
:
  m_pGame(pGame)
{
  // some init values

  rallyGoing=false;
  mmenuSel=0;
  mmenuOfs=0;
  rmenuSel=0;
  ixCar=0;
  ixMap=0;
	tyres=IRBRGame::TYRE_GRAVEL_DRY;
  updateRecords=false;
  driving=false;

  // scan for rally configs

  const CString srpFolder("Plugins\\RBRSingleRally");

  CString pat = srpFolder+"\\*.srp";
  WIN32_FIND_DATA find;
  HANDLE hFind=FindFirstFile(pat,&find);
  while (hFind!=INVALID_HANDLE_VALUE)
  {
    if (!(find.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY))
    {
      int pos;
      CString name(find.cFileName);
      if ((pos=name.ReverseFind('.'))>=0) name.Delete(pos,name.GetLength()-pos);
      Rally rally(name);
      rally.LoadConfig(srpFolder+"\\"+find.cFileName);
      rally.LoadBestTime();
      rallies.push_back(rally);
    }
    if (FindNextFile(hFind,&find)) continue;
    FindClose(hFind);
    break;
  }

  // scan for setup files

  for (int i=0;i<NUM_CARS;i++)
  {
    Car car(GetCarName(i));

    CString pat;
    pat.Format("SavedGames\\%i*.lsp",i);
    WIN32_FIND_DATA find;
    HANDLE hFind=FindFirstFile(pat,&find);
    while (hFind!=INVALID_HANDLE_VALUE)
    {
      car.setups.push_back(find.cFileName);
      if (FindNextFile(hFind,&find)) continue;
      FindClose(hFind);
      break;
    }
    cars.push_back(car);
  }

  // load stage records

  bestTimes.LoadStageRecords();

}

//-----------------------------------------------------------------------------

void SingleRallyPlugin::DrawFrontEndPage ()
{
  if (driving) // driver has cancelled the stage by 'exit to main menu'
  {
    DidNotFinish();
    NextMap(false);
    driving=false;
  }

  if (updateRecords)
  {
    bestTimes.UpdateRecord(curTime);
    updateRecords=false;
  }
	
  if (rallyGoing) DrawRallyMenu();
  else DrawMainMenu();

  // draw credits

	m_pGame->SetMenuColor(IRBRGame::MENU_TEXT);	
	m_pGame->SetFont(IRBRGame::FONT_SMALL);   
	m_pGame->WriteText(10.0f,450.0f,"RBRSingleRally plugin v0.3 by lowgoods");
}

//-----------------------------------------------------------------------------

void SingleRallyPlugin::DrawRallyMenu ()
{
  m_pGame->DrawBlackOut(0,240,640,480);

  float xofs = 15.0f;
  float yofs = 25.f;
  menuEntries=RMENU_NUM;

  char txt[256];
  wsprintf(txt,"Rally %s",rallies[ixRally].name);
	m_pGame->SetMenuColor(IRBRGame::MENU_HEADING);	
	m_pGame->SetFont(IRBRGame::FONT_BIG);
	
  m_pGame->WriteText(xofs,yofs,txt);
  yofs+=25;

  m_pGame->DrawSelection(0.0f,yofs-2+(static_cast<float>(rmenuSel)*21.0f),280.0f);
	m_pGame->SetMenuColor(IRBRGame::MENU_TEXT);

	for (int i=0;i<RMENU_NUM;++i)
	{
    switch ((RMENU)i)
    {
      case RMENU_CONTINUE:
      {
        if (rallies[ixRally].Finished()) wsprintf(txt,"Finish");
        else wsprintf(txt,"Continue: %s (%i of %i)",rallies[ixRally].stages[ixMap].name,ixMap+1,rallies[ixRally].stages.size());
        break;
      }
      case RMENU_CAR:
      {
        wsprintf(txt,"Car: %s",cars[ixCar].name);
        break;
      }
      /*
      case RMENU_WEATHER:
      {
        wsprintf(txt,"Weather: %s",GetWeatherName(rallies[ixRally].stages[ixMap].weather,true));
        break;
      }
      */
      case RMENU_TYRES:
      {
        wsprintf(txt,"Tyres: %s",GetTyreName(tyres));
        break;
      }
      case RMENU_SETUP:
      {
        sprintf(txt,"Setup: ");
        int ix=cars[ixCar].ixSetup;
        if (ix<0) sprintf(txt+strlen(txt),"Default");
        else
        {
          char ach[256];
          strcpy(ach,cars[ixCar].setups[ix]);
          char *pn = strchr(ach,'_');
          if (!pn) pn = ach;
          else pn+=1;
          char *ex = strrchr(ach,'.');
          if (ex) *ex=0;
          wsprintf(txt+strlen(txt),"%s",pn);
        }
        break;
      }
      case RMENU_RESET:
      {
        wsprintf(txt,"Reset Times");
        break;
      }
      case RMENU_ABORT:
      {
        wsprintf(txt,"Leave %s",rallies[ixRally].name);
        break;
      }
    }

    m_pGame->WriteText(xofs,yofs+(static_cast<float>(i)*21.0f),txt);    
  }

  DrawStandings();
}

//-----------------------------------------------------------------------------

void SingleRallyPlugin::DrawMainMenu ()
{
  m_pGame->DrawBlackOut(300.0f,0.0f,300.0f,480.0f);

  const float xofs = 73.0f;

	m_pGame->SetMenuColor(IRBRGame::MENU_HEADING);	
	m_pGame->SetFont(IRBRGame::FONT_BIG);
	m_pGame->WriteText(73.0f,49.0f,"Single Rally Training");

  if (rallies.size())
  {
  	m_pGame->DrawSelection(0.0f,68.0f+(static_cast<float>(mmenuSel+1)*21.0f),280.0f);
  	m_pGame->SetMenuColor(IRBRGame::MENU_TEXT);

	  for (int i=0;i<MAX_MMENU;++i)
	  {
      int ix = mmenuOfs+i;
      if (ix>=rallies.size()) break;
      m_pGame->WriteText(xofs,70.0f+(static_cast<float>(i+1)*21.0f),rallies[ix].name);    
    }
  }
  else
  {
  	m_pGame->SetMenuColor(IRBRGame::MENU_TEXT);
    m_pGame->WriteText(xofs,70.0f+(static_cast<float>(1)*21.0f),"No config files found");    
  }
}


//-----------------------------------------------------------------------------

void SingleRallyPlugin::DrawStandings ()
{
  Rally &curRally = rallies[ixRally];

  float ypos = 25;
  float xpos1=290,xpos2=430,xpos3=500,xpos4=570;

	m_pGame->SetMenuColor(IRBRGame::MENU_TEXT);	
	m_pGame->SetFont(IRBRGame::FONT_SMALL);   

  m_pGame->WriteText(xpos3,ypos,"Rally Record");
  m_pGame->WriteText(xpos4,ypos,"Stage Record");
  ypos+=25;

  char txt[256];
 	char tim[32];
  
  float totalDiff = 0;			
  float totalDiffRally = 0;			
  
  float totalRallyBest = 0;
  float totalStageBest = 0;

  for (int i=0;i<rallies[ixRally].stages.size();i++)
  {
    Stage &curStage = curRally.stages[i];


    // 1st col

    const char *pw = GetWeatherName(rallies[ixRally].stages[i].weather,true);

    m_pGame->SetColor(1.,1.,1.,1.);
    sprintf(txt,"%i) %s (%s)",i+1,rallies[ixRally].stages[i].name,pw);
    m_pGame->WriteText(xpos1,ypos,txt);

    // 2nd col
    
    if (rallies[ixRally].stages[i].Dnf()) sprintf(txt,"DNF");
    else sprintf(txt,"%s",FormatTimeString(tim,rallies[ixRally].stages[i].finish));
    m_pGame->WriteText(xpos2,ypos,txt);

    // 3rd col

    if (curStage.bestTime) totalRallyBest+=curStage.best.finish;

    float rallyBest = 0;
    
    if (rallies[ixRally].stages[i].bestTime)
    {
      pw = GetWeatherName(rallies[ixRally].stages[i].best.weather);

      if (rallies[ixRally].stages[i].finish==0)
      {
        sprintf(txt,"%s",FormatTimeString(tim,rallies[ixRally].stages[i].best.finish));
      }
      else
      {
        float diff = rallies[ixRally].stages[i].finish - rallies[ixRally].stages[i].best.finish;
        totalDiffRally += diff;
        if (diff<=0) m_pGame->SetColor(0.,1.,0.,1.);
        else m_pGame->SetColor(1.,0.,0.,1.);
        sprintf(txt,"%s",FormatTimeString(tim,diff,true));
      }
    }
    else
    {
      sprintf(txt,"%s",FormatTimeString(tim,rallyBest));
    }

    m_pGame->WriteText(xpos3,ypos,txt);

    // 4th col

    const BestTime* best=bestTimes.Get(rallies[ixRally].stages[i].index);
    
    if (best)
    {
      totalStageBest+=best->time;

      if (rallies[ixRally].stages[i].finish==0)
        sprintf(txt,"%s",FormatTimeString(tim,best->time));
      else
      {
        float diff = rallies[ixRally].stages[i].finish - best->time;
        totalDiff += diff;

        if (diff<=0) m_pGame->SetColor(0.,1.,0.,1.);
        else m_pGame->SetColor(1.,0.,0.,1.);
        
        sprintf(txt,"%s",FormatTimeString(tim,diff,true));
      }
      m_pGame->WriteText(xpos4,ypos,txt);
    }
    
    ypos+=17;
  }

  ypos += 5;

  m_pGame->SetColor(1.,1.,1.,1.);

  sprintf(txt,"Total");
  m_pGame->WriteText(xpos1,ypos,txt);

  if (rallies[ixRally].Dnf()) sprintf(txt,"DNF");
  else sprintf(txt,"%s",FormatTimeString(tim,rallies[ixRally].time));
  m_pGame->WriteText(xpos2,ypos,txt);

  if (curRally.NotStarted())
  {
    m_pGame->SetColor(1.,1.,1.,1.);
    m_pGame->WriteText(xpos3,ypos,FormatTimeString(tim,totalRallyBest,false));
    m_pGame->WriteText(xpos4,ypos,FormatTimeString(tim,totalStageBest,false));
  }
  else
  {
    if (totalDiffRally<=0) m_pGame->SetColor(0.,1.,0.,1.);
    else m_pGame->SetColor(1.,0.,0.,1.);
    sprintf(txt,"%s",FormatTimeString(tim,totalDiffRally,true));
    m_pGame->WriteText(xpos3,ypos,txt);

    if (totalDiff<=0) m_pGame->SetColor(0.,1.,0.,1.);
    else m_pGame->SetColor(1.,0.,0.,1.);
    sprintf(txt,"%s",FormatTimeString(tim,totalDiff,true));
    m_pGame->WriteText(xpos4,ypos,txt);
  }
}

//-----------------------------------------------------------------------------

void SingleRallyPlugin::HandleFrontEndEvents (char txtKeyboard, bool bUp, bool bDown, bool bLeft, bool bRight, bool bSelect )
{
  if (rallyGoing)
  {
    bool stageDriven = true;
    if (ixMap>=0) stageDriven = rallies[ixRally].stages[ixMap].driven;

	  if (bUp)
    {
      if (--rmenuSel<0) rmenuSel = menuEntries-1;
    }
  
    if (bDown)
    {
      if (++rmenuSel>=menuEntries) rmenuSel = 0;
    }
  
    if (bSelect)
    {
      switch (rmenuSel)
      {
        case RMENU_RESET:
        {
          Reset(ixRally);
          bestTimes.LoadStageRecords();
          break;
        }
        case RMENU_CONTINUE:
        {
          if (rallies[ixRally].Finished())
          {
            rallies[ixRally].SaveBestTime();
            rallyGoing=false;
            Reset(ixRally);
          }
          else
          {
            char sfile[MAX_PATH],*ps;
            int ix=cars[ixCar].ixSetup;
            if (ix<0) ps=0;
            else
            {
              wsprintf(sfile,"SavedGames\\%i",cars[ixCar].setups[ix]);
              ps=sfile;
            }
            m_pGame->StartGame(rallies[ixRally].stages[ixMap].index,ixCar,(IRBRGame::ERBRWeatherType)rallies[ixRally].stages[ixMap].weather,(IRBRGame::ERBRTyreTypes)tyres,ps);
          }
          break;
        }
        case RMENU_ABORT:
        {
          rallyGoing=false;
          break;
        }
      }
    }
    if (bLeft)
    {
      switch (rmenuSel)
      {
        case RMENU_SETUP: if (--cars[ixCar].ixSetup<-1) cars[ixCar].ixSetup=cars[ixCar].setups.size()-1; break;
        case RMENU_CONTINUE: NextMap(true); break;
        case RMENU_CAR: if (--ixCar<0) ixCar=NUM_CARS-1; break;
        //case RMENU_WEATHER: if (--rallies[ixRally].stages[ixMap].weather<0) rallies[ixRally].stages[ixMap].weather=NUM_WEATHER-1  ; break;
        case RMENU_TYRES: if (--tyres<0) tyres=NUM_TYRES-1  ; break;
      }
    }
    if (bRight)
    {
      switch (rmenuSel)
      {
        case RMENU_SETUP: if (++cars[ixCar].ixSetup>=cars[ixCar].setups.size()) cars[ixCar].ixSetup=-1; break;
        case RMENU_CONTINUE: NextMap(false); break;
        case RMENU_CAR: if (++ixCar>=NUM_CARS) ixCar=0; break;
        //case RMENU_WEATHER: if (++rallies[ixRally].stages[ixMap].weather>=NUM_WEATHER) rallies[ixRally].stages[ixMap].weather=0; break;
        case RMENU_TYRES: if (++tyres>=NUM_TYRES) tyres=0; break;
      }
    }
  }
  else // main menu
  {
    if (rallies.size())
    {
	    if (bUp)
      {
        mmenuSel--;
        if (mmenuSel<0) mmenuOfs--,mmenuSel++;
        if (mmenuSel+mmenuOfs<0)
        {
          if (rallies.size()>MAX_MMENU) mmenuOfs=rallies.size()-MAX_MMENU, mmenuSel=MAX_MMENU-1;
          else mmenuOfs=0,mmenuSel=rallies.size()-1;
        }
      }
	    if (bDown)
      {
        mmenuSel++;
        if (mmenuSel>=MAX_MMENU) mmenuOfs++,mmenuSel--;
        if (mmenuSel+mmenuOfs>=rallies.size()) mmenuOfs=0, mmenuSel=0;
      }
  	  if (bSelect)
      {
        ixRally=mmenuOfs+mmenuSel;
        rallyGoing=true;
        ixMap = -1;
        rmenuSel = RMENU_CONTINUE;
        NextMap(false);

        bestTimes.LoadStageRecords();
        rallies[ixRally].LoadBestTime();
      }
    }
  }
}

//------------------------------------------------------------------------------------------------//

void SingleRallyPlugin::ResetAll ()
{
  for (int i=0;i<rallies.size();i++) Reset(i);
}


void SingleRallyPlugin::Reset (int ix)
{
  rallies[ix].Reset();
  ixMap=0;
}

//------------------------------------------------------------------------------------------------//
// We have no animations that needs ticking, so we do nothing.

void SingleRallyPlugin::TickFrontEndPage (float fTimeDelta)
{
	
}

//------------------------------------------------------------------------------------------------//
// Is called when the player timer starts (after GO! or in case of a false start)

void SingleRallyPlugin::StageStarted (int iMap,const char* ptxtPlayerName,bool bWasFalseStart )
{
  driving=true;
}

//------------------------------------------------------------------------------------------------//
// Is called when a player passed a checkpoint 

void SingleRallyPlugin::CheckPoint (float fCheckPointTime,int iCheckPointID,const char* ptxtPlayerName)
{
  if (iCheckPointID<1||iCheckPointID>2) return;

  if (rallies[ixRally].stages[ixMap].bestTime)
  {
    float diff = fCheckPointTime - rallies[ixRally].stages[ixMap].best.check[iCheckPointID-1];
    DrawDiff(20,460,diff,true);
  }
}

//-----------------------------------------------------------------------------

void SingleRallyPlugin::NextMap (bool reverse)
{
  if (rallies[ixRally].Finished()) return;

  for (int i=0;i<rallies[ixRally].stages.size();i++)
  {
    if (reverse) --ixMap;
    else ++ixMap;

    if (ixMap<0) ixMap=rallies[ixRally].stages.size()-1;
    if (ixMap>=rallies[ixRally].stages.size()) ixMap=0;

    if (!rallies[ixRally].stages[ixMap].driven) break;
  }
}

//-----------------------------------------------------------------------------

void SingleRallyPlugin::DidNotFinish ()
{
  Rally& curRally = rallies[ixRally];
  Stage& curStage = curRally.stages[ixMap];

  curRally.invalidStage=true;
  curRally.time=0;
  curStage.driven=true;
  curStage.check[0]=0;
  curStage.check[1]=0;
  curStage.finish=0;
}

void SingleRallyPlugin::HandleResults (float fCheckPoint1,float fCheckPoint2,float fFinishTime,const char* ptxtPlayerName)
{
  driving=false;

  Rally& curRally = rallies[ixRally];
  Stage& curStage = curRally.stages[ixMap];

  curStage.driven=true;

  if (fFinishTime==0)
  {
    DidNotFinish();
  }
  else
  {
    if (!curRally.invalidStage) curRally.time += fFinishTime;
    curStage.driven=true;
    curStage.check[0]=fCheckPoint1;
    curStage.check[1]=fCheckPoint2;
    curStage.finish=fFinishTime;

    ZeroMemory(&curTime,sizeof curTime);
    curTime.car=ixCar;
    curTime.check[0]=curStage.check[0];
    curTime.check[1]=curStage.check[1];
    curTime.map=curStage.index;
    curTime.time=curStage.finish;
    strcpy(curTime.player,ptxtPlayerName);
    updateRecords=true;
  }

  ixDraw=ixMap;
  NextMap(false);
}

void SingleRallyPlugin::DrawDiff (float x,float y,float diff,bool inGame)
{
  char tim[32];
  if (diff>0) m_pGame->SetColor(1.,0.,0.,1.);
  else m_pGame->SetColor(0.,1.,0.,1.);
  CString txt;
  txt.Format("%s",FormatTimeString(tim,diff,true));
  
  if (inGame) m_pGame->WriteGameMessage(txt,10.,x,y);
  else m_pGame->WriteText(x,y,txt);
}

void SingleRallyPlugin::DrawResultsUI ()
{
  Rally& curRally = rallies[ixRally];
  Stage& curStage = curRally.stages[ixDraw];

  char txt[256];
  
  wsprintf(txt,"Results %s",rallies[ixRally].stages[ixMap].name);
	m_pGame->SetMenuColor(IRBRGame::MENU_HEADING);	
	m_pGame->SetFont(IRBRGame::FONT_BIG);
	m_pGame->WriteText(130.0f,49.0f,txt);

	m_pGame->SetFont(IRBRGame::FONT_SMALL);
	m_pGame->SetMenuColor(IRBRGame::MENU_TEXT);
	
	if (curStage.Dnf())
	{
		m_pGame->WriteText(200.0f,125.0f,"Did Not Finish");
	}
	else
	{
    const BestTime *best = bestTimes.Get(curStage.index);

    float xpos1=130,xpos2=230,xpos3=300,xpos4=370;
    float ypos = 100;
		char tim[32];

    m_pGame->WriteText(xpos3,ypos,"Rally Record");
    m_pGame->WriteText(xpos4,ypos,"Stage Record");
    ypos+=25;

    for (int c=0;c<2;c++)
    {
      m_pGame->SetColor(1,1,1,1);
		  sprintf(txt,"Check point %i",c+1);
      m_pGame->WriteText(xpos1,ypos,txt);

		  sprintf(txt,"%s",FormatTimeString(tim,curStage.check[c]));
      m_pGame->WriteText(xpos2,ypos,txt);

      if (curStage.bestTime)
      {
        float diff = curStage.check[c] - curStage.best.check[c];
        DrawDiff(xpos3,ypos,diff);
      }
      if (best)
      {
        float diff = curStage.check[c] - best->check[c];
        DrawDiff(xpos4,ypos,diff);
      }
      ypos+=21;
    }

    ypos+=5;

    m_pGame->SetColor(1,1,1,1);
		sprintf(txt,"Stage time");
    m_pGame->WriteText(xpos1,ypos,txt);

		sprintf(txt,"%s",FormatTimeString(tim,curStage.finish));
    m_pGame->WriteText(xpos2,ypos,txt);

    if (best)
    {
      if (curStage.bestTime)
      {
        float diff = curStage.finish - curStage.best.finish;
        DrawDiff(xpos3,ypos,diff);
      }

      float diff = curStage.finish - best->time;
      DrawDiff(xpos4,ypos,diff);
    }
  }

}

//-----------------------------------------------------------------------------
