#ifndef __TESTPLUGIN_H_INCLUDED
#define __TESTPLUGIN_H_INCLUDED

// You need to define this before the includes, or use /D __SUPPORT_PLUGIN__=1 
// in the C/C++ / Commandline project configuration
#include "IPlugin.h"
#include "IRBRGame.h"
#include "PluginHelpers.h"
#include "Gendef.h" // 2D Gfx header
#include "Settings.h"
#include "BestTimes.h"
#include "Rally.h"
#include "Car.h"
#include <stdio.h>
#include <vector>
#include <string>
using namespace std;

//------------------------------------------------------------------------------------------------//

class SingleRallyPlugin : public IPlugin
{

public:

	SingleRallyPlugin	(IRBRGame* pGame);	

	virtual const char* GetName () { return "Single Rally Training"; }

	virtual void DrawResultsUI( void );
	virtual void DrawFrontEndPage( void );
	virtual void HandleFrontEndEvents( char txtKeyboard, bool bUp, bool bDown, bool bLeft, bool bRight, bool bSelect );
	virtual void TickFrontEndPage( float fTimeDelta );
	virtual void StageStarted		( int iMap,  const char* ptxtPlayerName, bool bWasFalseStart );
	virtual void HandleResults		( float fCheckPoint1,  float fCheckPoint2, float fFinishTime, const char* ptxtPlayerName );
	virtual void CheckPoint	( float fCheckPointTime, int   iCheckPointID, const char* ptxtPlayerName );

private:
  
  enum { MAX_STAGES = 7 };
  enum { NUM_CARS = 8 };
  enum { NUM_RALLIES = 6 };
  enum { NUM_WEATHER = 3 };
  enum { NUM_TYRES = IRBRGame::NUM_TYRE_TYPES };
  enum { MAX_MMENU = 12 };
  
  enum RMENU { RMENU_CONTINUE, RMENU_CAR, /*RMENU_WEATHER,*/ RMENU_TYRES, RMENU_SETUP, RMENU_RESET, RMENU_ABORT, RMENU_NUM };
  int menuEntries;
  int mmenuSel,mmenuOfs;
  int rmenuSel;
  bool rallyGoing;

  BestTimes bestTimes;
  Settings set;
  vector<Rally> rallies;
  vector<Car> cars;

	IRBRGame*	m_pGame;		// Stores the pointer to the game interface.

  int ixRally;
  int ixMap;
  int ixCar;
	//int weather;
	int tyres;

  int ixDraw;
  CString playerName;
  bool updateRecords;
  BestTime curTime;

  bool driving;

private: // methods

  void NextMap (bool reverse);
  void Reset (int ix);
  void ResetAll ();
  void DrawStandings ();
  void DrawDiff (float x,float y,float diff,bool inGame=false);
  void DrawRallyMenu ();
  void DrawMainMenu ();
  void DidNotFinish ();
};

//-----------------------------------------------------------------------------

#endif